;	Copyright (c) 2003, Mike Berg aka N0QBH
;	N0QBH
;	Tested functional
;	Wavecomm Transmitter VFO Program
;	I2C Control 2.3 Ghz  -  2.5 Ghz
;	Reads tuner button input up or down
;	Displays 6 digits of frequency
;	For PIC16F627
;------------------------------------------------------------
        LIST    P=16F627
	ERRORLEVEL	-302
        include <P16F627.INC>
        __CONFIG        h'3d10'
;	
;-------------------------------------------------------------
;	Internal RC clock, all PORTA pins availible
;-------------------------------------------------------------
;               Port Set-up
;           0    1    2    3    4    5    6    7       bit
; port  A   In   In   In   In   Out  Out  Out  Out
;           SDA  SCL  UP   DN	   
;
;       B   Out  Out  Out  Out  Out  Out  Out  Out
;           E	 RS             LCD  LCD  LCD  LCD
;------------------------------------------------------------
	ORG	0			; start code
	goto	main
;------------------------------------------------------------
        cblock  0x20
		count_1
		count_2
		up_count
		down_count
                sda_count               ;loop down counter
                sda_data                ;I/O temp data
		lcd_data
		rf_freq_lo		;tuner freq lo byte
		rf_freq_hi		;tuner freq hi byte
		rf_disp_lo		;display working regs
		rf_disp_hi
		dp_buffer		;holds bin decimal pt data
		dp_data			;"     dec               "	
                hex_hi                  ;hex to decimal convert 
                hex_lo                  ;"                    "
                thousands               ;  1000's
                hundreds                ;  100's
                tens                    ;  10's
                ones                    ;  1's
		mirror_A
	endc
;------------------------------------------------------------
#define SDA             PORTA,0         ;bi directional data
#define SCL             PORTA,1         ;sync clock
#define	UP_BUTTON	mirror_A,2	;freq up
#define	DN_BUTTON	mirror_A,3	;freq down
#define E_LINE          PORTB,0         ;LCD E
#define RS_LINE         PORTB,1         ;LCD RS
;
rf_freq_lower_limit	equ	d'18400'	
rf_freq_upper_limit	equ	d'20000'
;
RF_ADDRESS_WRITE	equ	b'11000010'
RF_BYTE_4		equ	b'11001110'
RF_BYTE_5		equ	b'10000000'
;------------------------------------------------------------
main
        clrf    PORTA
        clrf    PORTB
        movlw   0x07
        movwf   CMCON                   ;compare module off
        bsf     STATUS,RP0              ;switch to bank 1
        movlw   b'00001111'             ;port A in/outputs
        movwf   TRISA
        movlw   b'00000000'             ;port B in/outputs
        movwf   TRISB
        movlw   b'10000001'             ;TMR0/4
        movwf   OPTION_REG
        bcf     STATUS,RP0              ;back to page 0
;
	movlw	low rf_freq_lower_limit
	movwf	rf_freq_lo
	movlw	high rf_freq_lower_limit
	movwf	rf_freq_hi		;preset freq regs
;
	call	lcd_delay		;wait 16 mS
        call    init_lcd
	clrf	up_count
	clrf	down_count
;
	call	enable_vfo
	goto	make_display
start_here
	call	long_delay		;wait 1/4 sec
;
check_up_button
	movfw	PORTA
	movwf	mirror_A
	btfss	UP_BUTTON
	goto	up_the_freq		;button pressed
	clrf	up_count		;button off, clear counter
	goto	check_down_button
up_the_freq
	incf	up_count,W
	sublw	d'8'
	bnc	up_fast
	incf	up_count,F
	call	frequency_increase_25
	goto	make_display	
up_fast
	call	frequency_increase_400
	goto	make_display
;
check_down_button
	btfss	DN_BUTTON
	goto	down_the_freq		;button pressed
	clrf	down_count		;button off, clear counter
	goto	start_here		;loop back
down_the_freq
	incf	down_count,W
	sublw	d'8'
	bnc	down_fast
	incf	down_count,F
	call	frequency_decrease_25
	goto	make_display
down_fast
	call	frequency_decrease_400
	goto	make_display
;
;------------------------------------------------------------
;	Total tuner count is 0 - 1600 in 125 Khz steps
;	Tuner count is divided by 2 to get 250 Khz decimal point steps
;	Tuner count is divided by 8 to give display 1 Mhz increments
;	Display is 2300.00 - 2500.00	Mhz
;------------------------------------------------------------
;
make_display
	movfw	rf_freq_lo
	movwf	rf_disp_lo
	movfw	rf_freq_hi
	movwf	rf_disp_hi		;xfer data to working regs
	clrc
	rrf	rf_disp_hi,F
	rrf	rf_disp_lo,F		;/2
	movfw	rf_disp_lo	
	andlw	b'00000011'
	movwf	dp_buffer		;0-3 - 00, 25, 50, 75
;
	clrc
	rrf	rf_disp_hi,F
	rrf	rf_disp_lo,F		;/4
	clrc
	rrf	rf_disp_hi,F
	rrf	rf_disp_lo,F		;/8
;
	clrf	dp_data			;convert dp_buffer to
	movfw	dp_buffer		;decimal in .25 increments
	bz	$+5	
	movlw	d'25'			;1=25, 2=50, 3=75
	addwf	dp_data,F
	decfsz	dp_buffer,F
	goto	$-2
;
	movfw	rf_disp_lo
	movwf	hex_lo
	movfw	rf_disp_hi
	movwf	hex_hi
	call	hd_convert
;
	call	front_half_display
;
	movlw	" "
	call	liter_d			;space
	movfw	thousands
	call	liter_d			;thousands
	movfw	hundreds
	call	liter_d			;hundreds
	movfw	tens
	call	liter_d			;tens
	movfw	ones
	call	liter_d			;ones
	movlw	"."
	call	liter_d			;decimal point
	movfw	dp_data
	movwf	hex_lo
	clrf	hex_hi
	call	hd_convert
	movfw	tens
	call	liter_d			;dp tens
	movfw	ones
	call	liter_d			;dp ones
;
	call	back_half_display
	movlw	" "
	call	liter_d
	movlw	" "
	call	liter_d
	movlw	"M"
	call	liter_d
	movlw	"h"
	call	liter_d
	movlw	"z"
	call	liter_d
	movlw	" "
	call	liter_d
	movlw	" "
	call	liter_d
	movlw	" "
	call	liter_d	
	call	adjust_vfo
	goto	start_here		;start over
;------------------------------------------------------------
;	I2C to VFO
enable_vfo
	movlw	RF_ADDRESS_WRITE	;write address
	movwf	sda_data
;
        call    Set_start		;send start condition
        call    tx_loop			;send address
        call    S_ack                   ;get slave ack
;
adjust_vfo
	movfw	rf_freq_hi
	movwf	sda_data
        call    tx_loop			;send hi byte
        call    S_ack                   ;get slave ack
;
	movfw	rf_freq_lo
	movwf	sda_data
        call    tx_loop			;send lo byte
        call    S_ack                   ;get slave ack
;
	movlw	RF_BYTE_4
	movwf	sda_data
        call    tx_loop			;send lo byte
        call    S_ack                   ;get slave ack
;
	movlw	RF_BYTE_5
	movwf	sda_data
        call    tx_loop			;send lo byte
        call    S_ack                   ;get slave ack
;
	return				;start over
;------------------------------------------------------------
frequency_increase_400
	movlw	d'32'
	goto	$+2
frequency_increase_25
	movlw	d'2'
	addwf	rf_freq_lo,F
	skpnc	
	incf	rf_freq_hi,F
	movlw	high rf_freq_upper_limit
	subwf	rf_freq_hi,W
	bnc	were_done_up
	movlw	low rf_freq_upper_limit
	subwf	rf_freq_lo,W
	skpc			
were_done_up
	return
	movlw	low rf_freq_upper_limit
	movwf	rf_freq_lo
	movlw	high rf_freq_upper_limit
	movwf	rf_freq_hi
	return				;hold at hi limit
frequency_decrease_400
	movlw	d'32'
	goto	$+2
frequency_decrease_25
	movlw	d'2'
	subwf	rf_freq_lo,F
	skpc	
	decf	rf_freq_hi,F		;rf_freq_lo < 2
	movfw	rf_freq_hi
	sublw	high rf_freq_lower_limit
;
	bnc	were_done_down		;rf_freq_hi > low limit
	movfw	rf_freq_lo				
	sublw	low rf_freq_lower_limit
	skpc			
were_done_down
	return				;rf_freq_lo > low limit
	movlw	low rf_freq_lower_limit
	movwf	rf_freq_lo
	movlw	high rf_freq_lower_limit
	movwf	rf_freq_hi
	return				;hold at lo limit
;------------------------------------------------------------
;               I2C Routines
;------------------------------------------------------------			
i2_clk_hi
        call    I2_delay                ;wait 30 uS
        bsf     STATUS,RP0              ;bank 1
        bsf     SCL                     ;clock in/hi
        bcf     STATUS,RP0              ;bank 0
	return
;------------------------------------------------------------
i2_clk_lo
        call	I2_delay		;wait 30 uS
        bsf     STATUS,RP0
        bcf     SCL                     ;clock output/lo
        bcf     STATUS,RP0
	movlw	b'11111101'
	andwf	PORTA,F			;insure SCL pin is low
	return
;------------------------------------------------------------
sda_in_hi
        bsf     STATUS,RP0              ;bank 1
        bsf     SDA                     ;A:0 input/hi
        bcf     STATUS,RP0              ;bank 0
        return
;
sda_out_lo
        bsf     STATUS,RP0              ;switch to bank 1
        bcf     SDA                     ;A:0 output/low
        bcf     STATUS,RP0              ;back to page 0
	movlw	b'11111110'
	andwf	PORTA,F			;insure SDA pin is low
        return
;------------------------------------------------------------
Set_start
	call	i2_clk_hi		;sets start condition
	call	I2_delay
	call	sda_out_lo
	return
;------------------------------------------------------------
S_ack
        call	i2_clk_lo
        call    sda_in_hi
        call    i2_clk_hi               ;cycle clock
        btfsc   SDA
        goto    $-1
        return
;------------------------------------------------------------
M_ack
        call	i2_clk_lo
        call    sda_out_lo
        call	i2_clk_hi
        return
;------------------------------------------------------------
M_stop
        call	i2_clk_lo
        call    sda_out_lo
        call    i2_clk_hi               ;clock cycle,
        call    sda_in_hi               ;stop condition
        return
;------------------------------------------------------------
tx_loop
        movlw   0x08
        movwf   sda_count
loop_1
        call    i2_clk_lo               ;clock low
        rlf     sda_data,F
        bc      $+3
        call    sda_out_lo
        goto    $+2
        call    sda_in_hi
        call	i2_clk_hi
        decfsz  sda_count,F
        goto    loop_1
        return
;------------------------------------------------------------
;               hex to decimal convert hex registers (hi-lo)
;------------------------------------------------------------
hd_convert
        movlw   0x30                    ;all zero ascii
        movwf   ones
        movwf   tens
        movwf   hundreds
        movwf   thousands
thou_hd
        movlw   0x03
        subwf   hex_hi,W
        bnc     hund_hd
        movlw   0x03
        subwf   hex_hi,F
        movlw   0xe8
        subwf   hex_lo,W
        bnc     thou_check
do_lo1
        movlw   0xe8
        subwf   hex_lo,F
        incf    thousands,F             ;1000's + 1
        goto    thou_hd
thou_check
        movfw   hex_hi
        skpnz
        goto    add_back_1k
        decf    hex_hi,F
        goto    do_lo1
add_back_1k
        movlw   0x03
        addwf   hex_hi,F
hund_hd
        movlw   0x64
        subwf   hex_lo,W
        bnc     hund_check
do_lo2
        movlw   0x64
        subwf   hex_lo,F
        incf    hundreds,F              ;100's + 1
        goto    hund_hd
hund_check
        movfw   hex_hi
        skpnz
        goto    tens_hd
        decf    hex_hi,F
        goto    do_lo2
tens_hd
        movlw   0x0a
        subwf   hex_lo,F
        bnc     add_back_10
        incf    tens,F                  ;10's + 1
        goto    tens_hd
add_back_10
        movlw   0x0a
        addwf   hex_lo,F
        movfw   hex_lo
        addwf   ones,F                  ;remainder = 1's
        return
;------------------------------------------------------------
;               LCD Routines
;------------------------------------------------------------
init_lcd
        bcf     E_LINE                  ;E line low
        bcf     RS_LINE                 ;RS low, control
        call    lcd_delay               ;delay 15 mS
        movlw   0x30                    ;8-bit, 5X7 mode
        movwf   lcd_data
        call    flipbit
        call    pulse
	call	lcd_delay
        movlw   0x20                    ;4-bit, 5x7 mode
        movwf   lcd_data
        call    flipbit
        call    pulse
	call	lcd_delay
        movlw   0x28                    ;4-bit, 5x7 mode
        call    liter_c                 ;send both nybbles
        movlw   0x0c                    ;display on, cursor off
        call    liter_c
        movlw   0x06                    ;increment mode
        call    liter_c
        movlw   0x01                    ;clear display
        call    liter_c
 	call	lcd_delay
        return
;------------------------------------------------------------
front_half_display
        movlw   0x80
        call    liter_c                 ;
        return
;------------------------------------------------------------
back_half_display
        movlw   0xc0
        call    liter_c                 ;
        return
;------------------------------------------------------------
;		sends control byte to LCD
liter_c					
        movwf   lcd_data                ;store w in lcd_data
	bcf	RS_LINE			;control
        call    d_125uS
        call    flipbit                 ;hi nibble to PORTB
	call	pulse
        swapf   lcd_data,F              ;swap MS and LS nybbles
        call    flipbit                 ;output what was LS nybble
	call	pulse
	return
;------------------------------------------------------------
;		sends W to LCD
liter_d					
        movwf   lcd_data                ;store w in lcd_data
	bsf	RS_LINE			;data
        call    d_125uS
        call    flipbit                 ;hi nibble to PORTB
	call	pulse
        swapf   lcd_data,F              ;swap MS and LS nybbles
        call    flipbit                 ;output what was LS nybble
	call	pulse
	return
;------------------------------------------------------------
;		clears upper nibble of PORTB then writes
;		upper nibble of lcd_data to PORTB
flipbit
	movlw	0x0f
	andwf	PORTB,F
	movfw	lcd_data
	andlw	0xf0
	iorwf 	PORTB,F
	return
;------------------------------------------------------------
pulse
        bsf     E_LINE                  ;pulse E line
        nop                             ;delay
        bcf     E_LINE
        call    d_125uS                 ;delay 125 uS
        return
;------------------------------------------------------------
clear_LCD_display
        movlw   0x01                    ;clear display
        call    liter_c                 ;send to display
        call    lcd_delay               ;allow 15 mS to clear
        return
;------------------------------------------------------------
lcd_delay
        movlw   0x0f                     ;16 mS delay
        movwf   count_1
        goto    $+3
long_delay
        movlw   0xff
        movwf   count_1                  ;256 mS delay
        call    short_delay
        decfsz  count_1,F
        goto    $-2
        return
;
d_125uS
	movlw	d'30'
	goto	$+2
short_delay
        movlw   0xff                     ;1 mS delay
        movwf   count_2
        nop
        decfsz  count_2,F
        goto    $-2
        return
;------------------------------------------------------------
I2_delay
	movlw	d'8'
	movwf	count_2
	decfsz	count_2,F
	goto	$-1
	return
;------------------------------------------------------------
        end
;
Copyright (c) 2003, Mike Berg aka N0QBH
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 
Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 
Neither the name of the <ORGANIZATION> nor the names of its contributors 
may be used to endorse or promote products derived from this software without 
specific prior written permission. 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.
